//------------------------------------------------------------------------------
//
// File:    apsconfig.cc
//
// Purpose: Config parameters
//   
// (c) 2004 Plekhanov Andrey
//
// Initial version 0.1 06.03.2004
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//------------------------------------------------------------------------------

#include "apsconfig.h"

namespace aps {

  namespace apslib {

//------------------------------- APSAbsParameter ------------------------------

APSAbsParameter :: APSAbsParameter( const std::string & aName ) :
                                    Name( aName )
{
}

APSAbsParameter :: ~APSAbsParameter( void )
{
}

//------------------------------- APSParameter ---------------------------------

APSParameter :: APSParameter( const std::string & aName, const PARAM_TYPE aType,
                              const APSParameter * apNextParameter ) :
                APSAbsParameter( aName ), Type( aType ), pNextParameter( apNextParameter )
{
}

APSParameter :: ~APSParameter( void )
{
}

//------------------------------- APSIntegerParameter ---------------------------------

APSIntegerParameter :: APSIntegerParameter( const std::string & aName, const int aValue,
                                            const APSIntegerParameter * apNextParameter ) :
                       APSAbsParameter( aName ), Value( aValue ), pNextParameter( apNextParameter )
{
}

APSIntegerParameter :: ~APSIntegerParameter( void )
{
}

//------------------------------- APSDoubleParameter ---------------------------------

APSDoubleParameter :: APSDoubleParameter( const std::string & aName, const double aValue,
                                          const APSDoubleParameter * apNextParameter ) :
                        APSAbsParameter( aName ), Value( aValue ), pNextParameter( apNextParameter )
{
}

APSDoubleParameter :: ~APSDoubleParameter( void )
{
}

//------------------------------- APSStringParameter ---------------------------------

APSStringParameter :: APSStringParameter( const std::string & aName, const std::string & aValue,
                                          const APSStringParameter * apNextParameter ) :
                        APSAbsParameter( aName ), Value( aValue ), pNextParameter( apNextParameter )
{
}

APSStringParameter :: ~APSStringParameter( void )
{
}

//------------------------------- APSConfig ---------------------------------

APSConfig :: APSConfig( void ) : pFirstParameter( 0 ), pIntegerFirstParameter( 0 ),
                                 pDoubleFirstParameter( 0 ), pStringFirstParameter( 0 )
{
}

APSConfig :: ~APSConfig( void )
{
  const APSParameter        * pTmpParameter;
  const APSParameter        * pNextParameter;
  const APSIntegerParameter * pTmpIntegerParameter;
  const APSIntegerParameter * pNextIntegerParameter;
  const APSDoubleParameter  * pTmpDoubleParameter;
  const APSDoubleParameter  * pNextDoubleParameter;
  const APSStringParameter  * pTmpStringParameter;
  const APSStringParameter  * pNextStringParameter;

  pTmpParameter = pFirstParameter;

  while( pTmpParameter ) {
    pNextParameter = pTmpParameter->GetNextParameterPtr();

    delete pTmpParameter;

    pTmpParameter = pNextParameter;
  }

  pTmpIntegerParameter = pIntegerFirstParameter;

  while( pTmpIntegerParameter ) {
    pNextIntegerParameter = pTmpIntegerParameter->GetNextParameterPtr();

    delete pTmpIntegerParameter;

    pTmpIntegerParameter = pNextIntegerParameter;
  }

  pTmpDoubleParameter = pDoubleFirstParameter;

  while( pTmpDoubleParameter ) {
    pNextDoubleParameter = pTmpDoubleParameter->GetNextParameterPtr();

    delete pTmpDoubleParameter;

    pTmpDoubleParameter = pNextDoubleParameter;
  }

  pTmpStringParameter = pStringFirstParameter;

  while( pTmpStringParameter ) {
    pNextStringParameter = pTmpStringParameter->GetNextParameterPtr();

    delete pTmpStringParameter;

    pTmpStringParameter = pNextStringParameter;
  }
}

void APSConfig :: AddIntegerParameter( const std::string & Name, const int Value )
{
  APSIntegerParameter * pNewParameter = new APSIntegerParameter( Name, Value, pIntegerFirstParameter );

  pIntegerFirstParameter = pNewParameter;
}

void APSConfig :: AddDoubleParameter( const std::string & Name, const double Value )
{
  APSDoubleParameter * pNewParameter = new APSDoubleParameter( Name, Value, pDoubleFirstParameter );

  pDoubleFirstParameter = pNewParameter;
}

void APSConfig :: AddStringParameter( const std::string & Name, const std::string & Value )
{
  APSStringParameter * pNewParameter = new APSStringParameter( Name, Value, pStringFirstParameter );

  pStringFirstParameter = pNewParameter;
}

const APSIntegerParameter * APSConfig :: FindIntegerParameter( const std::string & Name ) const
{
  const APSIntegerParameter * pTmpParameter = pIntegerFirstParameter;

  while( pTmpParameter ) {
    if( pTmpParameter->GetName() == Name ) {
      return( pTmpParameter );
    }

    pTmpParameter = pTmpParameter->GetNextParameterPtr();
  }

  return( 0 );
}

const APSDoubleParameter * APSConfig :: FindDoubleParameter( const std::string & Name ) const
{
  const APSDoubleParameter * pTmpParameter = pDoubleFirstParameter;

  while( pTmpParameter ) {
    if( pTmpParameter->GetName() == Name ) {
      return( pTmpParameter );
    }

    pTmpParameter = pTmpParameter->GetNextParameterPtr();
  }

  return( 0 );
}

const APSStringParameter * APSConfig :: FindStringParameter( const std::string & Name ) const
{
  const APSStringParameter * pTmpParameter = pStringFirstParameter;

  while( pTmpParameter ) {
    if( pTmpParameter->GetName() == Name ) {
      return( pTmpParameter );
    }

    pTmpParameter = pTmpParameter->GetNextParameterPtr();
  }

  return( 0 );
}

int APSConfig :: IfExistsParameter( const std::string & Name ) const
{
  if( FindIntegerParameter( Name ) ) {
    return( 1 );
  }
  else {
    if( FindDoubleParameter( Name ) ) {
      return( 1 );
    }
    else {
      if( FindStringParameter( Name ) ) {
        return( 1 );
      }
    }
  }

  return( 0 );
}

int APSConfig :: AddParameter( const std::string & Name, const PARAM_TYPE Type )
{
  APSParameter * pNewParameter;
  int            RetCode = 0;

  if( ( Type == PARAM_INTEGER ) || ( Type == PARAM_DOUBLE ) || ( Type == PARAM_STRING ) ) {
    if( !FindParameter( Name ) ) {
      pNewParameter = new APSParameter( Name, Type, pFirstParameter );

      pFirstParameter = pNewParameter;
    }
    else {
      RetCode = 2;
    }
  }
  else {
    RetCode = 1;
  }

  return( RetCode );
}

const APSParameter * APSConfig :: FindParameter( const std::string & Name ) const
{
  const APSParameter * pTmpParameter = pFirstParameter;

  while( pTmpParameter ) {
    if( pTmpParameter->GetName() == Name ) {
      return( pTmpParameter );
    }

    pTmpParameter = pTmpParameter->GetNextParameterPtr();
  }

  return( 0 );
}

int APSConfig :: GetIntegerValue( const std::string & Name, int & Value ) const
{
  const APSIntegerParameter * pTmpParameter = pIntegerFirstParameter;

  while( pTmpParameter ) {
    if( pTmpParameter->GetName() == Name ) {
      Value = pTmpParameter->GetValue();
      return( 0 );
    }

    pTmpParameter = pTmpParameter->GetNextParameterPtr();
  }

  return( 1 );
}

int APSConfig :: GetDoubleValue( const std::string & Name, double & Value ) const
{
  const APSDoubleParameter * pTmpParameter = pDoubleFirstParameter;

  while( pTmpParameter ) {
    if( pTmpParameter->GetName() == Name ) {
      Value = pTmpParameter->GetValue();
      return( 0 );
    }

    pTmpParameter = pTmpParameter->GetNextParameterPtr();
  }

  return( 1 );
}

int APSConfig :: GetStringValue( const std::string & Name, std::string & Value ) const
{
  const APSStringParameter * pTmpParameter = pStringFirstParameter;

  while( pTmpParameter ) {
    if( pTmpParameter->GetName() == Name ) {
      Value = pTmpParameter->GetValue();
      return( 0 );
    }

    pTmpParameter = pTmpParameter->GetNextParameterPtr();
  }

  return( 1 );
}

}}

//---------------------------- End of file ---------------------------











/*int APSParameter :: GetDoubleValue( double &aValue ) const
{
  int RetCode;

  if( Type == PARAM_DOUBLE ) {
    if( sscanf( Value, "%lf", &aValue ) != 1 ) {
      RetCode = 2;
    }
  }
  else {
    RetCode = 1;
  }

  return( RetCode ); 
}

int Parameter :: GetIntValue( int Value ) const
{
  int aValue;

  if( Type != PARAM_INTEGER ) 
    MRLExit( MN_COMMON, EM_WRONG_TYPE, "Parameter :: GetIntValue", GetName() );

  if( sscanf( Value, "%d", &aValue ) != 1 )
    MRLExit( MN_COMMON, EM_CONVERT, "Parameter :: GetIntValue", GetName() );

  return( aValue );
}

const char * Parameter :: GetStringValue( void ) const
{
  if( Type != PARAM_STRING )
    MRLExit( MN_COMMON, EM_WRONG_TYPE, "Parameter :: GetStringValue", GetName() );

  return( Value );
}

int Parameter :: GetBoolValue( void ) const
{
  int aValue;

  if( Type != PARAM_BOOLEAN ) 
    MRLExit( MN_COMMON, EM_WRONG_TYPE, "Parameter :: GetBoolValue", GetName() );

  aValue = CheckBoolParam( Value );

  if( aValue == -1 )
    MRLExit( MN_COMMON, EM_CONVERT, "Parameter :: GetBoolValue", GetName() );

  return( aValue );
}

int Parameter :: SetValue( double aValue )
{
  char TmpValue[ MAX_STRING_LENGTH ]; 

  if( Type == PARAM_DOUBLE ) {
    sprintf( TmpValue, "%lf", aValue );
    delete [] Value;
    Value = newStr( TmpValue );
    return( 1 );
  }
  else
    return( 0 );
}

int Parameter :: SetValue( int    aValue )
{
  char TmpValue[ MAX_STRING_LENGTH ]; 

  if( Type == PARAM_INTEGER ) {
    sprintf( TmpValue, "%d", aValue );
    delete [] Value;
    Value = newStr( TmpValue );
    return( 1 );
  }
  else
    return( 0 );
}

int Parameter :: SetValue( const char * aValue )
{
  switch( Type ) {
    case PARAM_STRING:
      delete [] Value;
      Value = newStr( aValue );
    return( 1 );
    case PARAM_BOOLEAN:
      if( CheckBoolParam( aValue ) != -1 ) {
        delete [] Value;
        Value = newStr( aValue );
        return( 1 );
      }
    return( 0 );
  }
    
  return( 0 );
}

int Parameter :: ChangeValue( const char * aValue )
{
  if( CheckValue( aValue, Type ) )
    return( 1 );

  delete [] Value;
  Value = newStr( aValue );

  return( 0 );
}

int Parameter :: CheckValue( const char * aValue, int aType )
{
  double DoubleValue;
  int    IntValue;
  char   StringValue[ MAX_STRING_LENGTH ];

  switch( aType ) {
    case PARAM_DOUBLE:
      if( sscanf( aValue, "%lf", &DoubleValue ) != 1 )
        return( 1 );
    break;

    case PARAM_INTEGER:
      if( sscanf( aValue, "%d%s", &IntValue, StringValue ) != 1 )
        return( 1 );
    break;

    case PARAM_STRING:
    break;

    case PARAM_BOOLEAN:
      if( CheckBoolParam( aValue ) == -1 )
        return( 1 );
    break;

    default:
      return( 1 );
  }

  return( 0 );
}

int Parameter :: GetSource( void ) const
{
  return( Source );
}

void Parameter :: SetSource( int aSource )
{
  Source = aSource;
}*/






/*const int INIT_PARAM_NUMBER  = 20;
const int DELTA_PARAM_NUMBER = 10;

MRLConfig :: MRLConfig( void )
{
  pParamColl = NULL;
}

MRLConfig :: ~MRLConfig( void )
{
  int         i;
  Parameter * pParameter;

  for( i = 0; i < pParamColl->GetCount(); i++ ) {
    pParameter = pParamColl->GetParameterPtr( i );
    delete pParameter;
  }

  if( NULL != pParamColl )
    delete pParamColl;

  if( ConfigFileName != NULL ) {
    delete [] ConfigFileName;
  }

  if( DefaultConfigFileName != NULL ) {
    delete [] DefaultConfigFileName;
  }
}

int MRLConfig :: ParsingStr( const char * String, const char * FileName )
{
  char        ParamType[ MAX_STRING_LENGTH ];
  char        ParamName[ MAX_STRING_LENGTH ];
  char        ParamValue[ MAX_STRING_LENGTH ];
  char        NullParam[ MAX_STRING_LENGTH ];
  Parameter * pParameter;

  if( ( String[ 0 ] == '#') || ( !strcmp("\n", String ) ) )
    return( 1 );

  if( sscanf( String, "%s%s%s%s", ParamType, ParamName, ParamValue, NullParam ) != 3 )
    MRLExit( MN_MRLCONFIG, EM_READ_ERROR, FileName, String );

  if( !strcmp( "double", ParamType ) ) {
    switch( ChangeParameter( ParamName, ParamValue, PARAM_DOUBLE ) ) {
      case 0:
        pParameter = GetParameter( ParamName );

        if( NULL != pParameter ) {
          pParameter->SetSource( 1 );
        }
        else {
          MRLExit( MN_MRLCONFIG, EM_UNKNOWN_PARAM, ParamName );  
        }
      break;
      case 1:
        MRLExit( MN_MRLCONFIG, EM_UNKNOWN_PARAM, ParamName );
      case 2:
        MRLExit( MN_MRLCONFIG, EM_WRONG_TYPE_PARAM, ParamName );
      case 3:
        MRLExit( MN_MRLCONFIG, EM_WRONG_VALUE, ParamName );
      default:
        MRLExit( MN_COMMON, EM_UNKNOWN,"MRLConfig :: SetAllParameters",
          ParamName );
    }
  }
  else {
    if( !strcmp( "int", ParamType ) ) {
      switch( ChangeParameter( ParamName, ParamValue, PARAM_INTEGER ) ) {
        case 0:
          pParameter = GetParameter( ParamName );

          if( NULL != pParameter ) {
            pParameter->SetSource( 1 );
          }
          else {
            MRLExit( MN_MRLCONFIG, EM_UNKNOWN_PARAM, ParamName );  
          }
        break;
        case 1:
          MRLExit( MN_MRLCONFIG, EM_UNKNOWN_PARAM, ParamName );
        case 2:
          MRLExit( MN_MRLCONFIG, EM_WRONG_TYPE_PARAM, ParamName );
        case 3:
          MRLExit( MN_MRLCONFIG, EM_WRONG_VALUE, ParamName );
        default:
          MRLExit( MN_COMMON, EM_UNKNOWN,"MRLConfig :: SetAllParameters",
            ParamName );
      }
    }
    else {
      if( !strcmp( "string", ParamType ) ) {
        switch( ChangeParameter( ParamName, ParamValue, PARAM_STRING ) ) {
          case 0:
            pParameter = GetParameter( ParamName );

            if( NULL != pParameter ) {
              pParameter->SetSource( 1 );
            }
            else {
              MRLExit( MN_MRLCONFIG, EM_UNKNOWN_PARAM, ParamName );  
            }
          break;
          case 1:
            MRLExit( MN_MRLCONFIG, EM_UNKNOWN_PARAM, ParamName );
          case 2:
            MRLExit( MN_MRLCONFIG, EM_WRONG_TYPE_PARAM, ParamName );
          case 3:
            MRLExit( MN_MRLCONFIG, EM_WRONG_VALUE, ParamName );
          default:
            MRLExit( MN_COMMON, EM_UNKNOWN,"MRLConfig :: SetAllParameters",
              ParamName );
        }
      }
      else {
        if( !strcmp( "bool", ParamType ) ) {
          switch( ChangeParameter( ParamName, ParamValue, PARAM_BOOLEAN ) ) {
            case 0:
               pParameter = GetParameter( ParamName );

              if( NULL != pParameter ) {
                pParameter->SetSource( 1 );
              }
              else {
                MRLExit( MN_MRLCONFIG, EM_UNKNOWN_PARAM, ParamName );  
              }
            break;
            case 1:
              MRLExit( MN_MRLCONFIG, EM_UNKNOWN_PARAM, ParamName );
            case 2:
              MRLExit( MN_MRLCONFIG, EM_WRONG_TYPE_PARAM, ParamName );
            case 3:
              MRLExit( MN_MRLCONFIG, EM_WRONG_VALUE, ParamName );
            default:
              MRLExit( MN_COMMON, EM_UNKNOWN,"MRLConfig :: SetAllParameters",
                ParamName );
          }
        }
        else {
          MRLExit( MN_MRLCONFIG, EM_WRONG_TYPE_PARAM, ParamName );
        }
      }
    }
  }

  return( 0 );
}

int MRLConfig :: SetAllParameters( const char * FileName )
{
  MRLIStrFile * pFile;
  char        aString[ MAX_STRING_LENGTH ];

  pFile = new MRLIStrFile( FileName );

  if( pFile == NULL )
    MRLExit( MN_COMMON, EM_ALLOCATE,"MRLConfig :: SetAllParameters", "pFile" );

  if( pFile->Open() ) {
    delete pFile;
    return( 2 );
  }

  while( !pFile->GetStr( aString, MAX_STRING_LENGTH ) ) {
    ParsingStr( aString, FileName );
  }

  pFile->Close();
  delete pFile;
  return( 0 );
}

int MRLConfig :: Init( const char * aConfigFileName,
                       const char * aDefaultConfigFileName )
{
  pParamColl = new ParameterCollection( INIT_PARAM_NUMBER, DELTA_PARAM_NUMBER );

  ConfigFileName        = newStr( aConfigFileName );
  DefaultConfigFileName = newStr( aDefaultConfigFileName );

  if( NULL == pParamColl )
    return( 1 );

  SetDefaultParameters();

  if( NULL != aDefaultConfigFileName )
    if( SetAllParameters( aDefaultConfigFileName ) == 1 )
      return( 1 );

  if( NULL != aConfigFileName )
    if( SetAllParameters( aConfigFileName ) == 1 )
      return( 1 );

  return( 0 );
}

void MRLConfig :: SetDefaultParameters( void )
{
}

Parameter * MRLConfig :: GetParameter( const char * Name ) const
{
  int i;

  for( i = 0; i < pParamColl->GetCount(); i++ ) {
    if( !strcmp( Name, pParamColl->GetParameterPtr( i )->GetName() ) ) {
      return pParamColl->GetParameterPtr(i);
    }
  }

  return( NULL );
}

double MRLConfig :: GetDoubleValue( const char * Name ) const
{
  Parameter * pParameter;

  pParameter = GetParameter( Name );

  if( NULL == pParameter )
    MRLExit( MN_MRLCONFIG, EM_UNKNOWN_PARAM, Name );

  return( pParameter->GetDoubleValue() );
}

int MRLConfig :: GetIntValue( const char * Name ) const
{
  Parameter * pParameter;

  pParameter =  GetParameter( Name );

  if( NULL == pParameter )
    MRLExit( MN_MRLCONFIG, EM_UNKNOWN_PARAM, Name );

  return( pParameter->GetIntValue() );
}

const char * MRLConfig :: GetStringValue( const char * Name ) const
{
  Parameter * pParameter;

  pParameter = GetParameter( Name );

  if( NULL == pParameter )
    MRLExit( MN_MRLCONFIG, EM_UNKNOWN_PARAM, Name );

  return( pParameter->GetStringValue() );
}

int MRLConfig :: GetBoolValue( const char * Name ) const
{
  Parameter * pParameter;

  pParameter = GetParameter( Name );

  if( NULL == pParameter )
    MRLExit( MN_MRLCONFIG, EM_UNKNOWN_PARAM, Name );

  return( pParameter->GetBoolValue() );
}

int MRLConfig :: SetParameterValue( const char * Name, double aValue )
{
  Parameter * pParameter;

  pParameter = GetParameter( Name );

  if( NULL != pParameter ) {
    if( pParameter->SetValue( aValue ) ) {
      pParameter->SetSource( 2 );
      return( 1 );
    }  
    else
      return( 0 );
  }
  return( 0 );
}

int MRLConfig :: SetParameterValue( const char * Name, int aValue )
{
  Parameter * pParameter;

  pParameter = GetParameter( Name );

  if( NULL != pParameter ) {
    if( pParameter->SetValue( aValue ) ) {
      pParameter->SetSource( 2 );
      return( 1 );
    }  
    else
      return( 0 );
  }
  return( 0 );
}

int MRLConfig :: SetParameterValue( const char * Name, const char * aValue )
{
  Parameter * pParameter;

  pParameter = GetParameter( Name );

  if( NULL != pParameter ) {
    if( pParameter->SetValue( aValue ) ) {
      pParameter->SetSource( 2 );
      return( 1 );
    }  
    else
      return( 0 );
  }
  return( 0 );
}

int MRLConfig :: AddParameter( const char * aName, const char * aValue, int aType )
{
  Parameter * pParameter;

  pParameter = GetParameter( aName );

  if( NULL == pParameter ) {
    if( Parameter :: CheckValue( aValue, aType ) )
      return( 1 );

    pParameter = new Parameter( aName, aValue, aType );
    if( NULL == pParameter )
      return( 1 );

    if( pParameter->ChangeValue( aValue ) )
      return( 1 );

    pParamColl->AddParameter( pParameter );
  }

  return( 0 );
}

int MRLConfig :: ChangeParameter( const char * aName, const char * aValue, int aType )
{
  Parameter * pParameter;

  pParameter = GetParameter( aName );

  if( NULL == pParameter )
    return( 1 );

  if( pParameter->GetType() != aType )
    return( 2 );

  if( pParameter->ChangeValue( aValue ) )
    return( 3 );

  return( 0 );
}

int MRLConfig :: ChangeParameter( const char * aName, const char * aValue )
{
  Parameter * pParameter;

  pParameter = GetParameter( aName );

  if( NULL == pParameter )
    return( 1 );

  if( pParameter->ChangeValue( aValue ) )
    return( 3 );

  return( 0 );
}

int MRLConfig :: GetParameterSource( const char * Name ) const
{
  Parameter * pParameter;

  pParameter = GetParameter( Name );

  if( NULL != pParameter ) {
    switch( pParameter->GetSource() ) {
      case 0:
        return( PARAM_DEFAULT );
      case 1:
        return( PARAM_CONFIG );
      case 2:
        return( PARAM_SET );  
      default:
        MRLExit( MN_COMMON, EM_UNKNOWN,"MRLConfig :: GetParameterSource", "Source" );
    }
  }        
  
  return( PARAM_NOT_FOUND );
}*/
