//------------------------------------------------------------------------------
//
// File:    xlifeModuleAppl.cc
//
// Purpose: Module for XLife application.
//   
// (c) Plekhanov Andrey
//
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//------------------------------------------------------------------------------
#include <iostream>
#include <sstream>

#include "apsreadcfg.h"
#include "xlifeModuleAppl.h"
#include "xlifeSubModule.h"
#include "xlifeConfigReadSubModule.h"
#include "xlifeConfig.h"

namespace aps {

  namespace xlifeappl {

using apslib::APSReadConfig;

//======================= XLifeModuleAppl ==========================

XLifeModuleAppl :: XLifeModuleAppl( const std::string & aProjectFilePath,
                                    const std::string & aModuleName,
                                    const int X_Version,
                                    const int Y_Version,
                                    const int Z_Version,
                                    const int FirstCopyRightYear,
                                    const int CopyRightYear,
                                    const apslib::APS_PROGRAM_VERSION_TYPE VersionType ) :
                    APSMainModule( aModuleName, X_Version, Y_Version, Z_Version,
                                   FirstCopyRightYear, CopyRightYear, VersionType ),
                                   ProjectFilePath( aProjectFilePath )
{
  pXLifeSubModule           = new XLifeSubModule( this );
  pXLifeConfigReadSubModule = new XLifeConfigReadSubModule( this );

  BorderField = 128;
  Delta       = 10000;
  Width       = 1024;
  Height      = 768;
  InitRandom  = 1;

  OriginBorderField = XLIFE_APPL_PARAM_DEFAULT;
  OriginDelta       = XLIFE_APPL_PARAM_DEFAULT;
  OriginWidth       = XLIFE_APPL_PARAM_DEFAULT;
  OriginHeight      = XLIFE_APPL_PARAM_DEFAULT;
  OriginInitRandom  = XLIFE_APPL_PARAM_DEFAULT;
}

XLifeModuleAppl :: ~XLifeModuleAppl( void )
{
  delete pXLifeConfigReadSubModule;
  delete pXLifeSubModule;
}

bool XLifeModuleAppl :: IfWidth( void ) const
{
  if( OriginWidth == XLIFE_APPL_PARAM_DEFAULT ) {
    return( false );
  }

  return( true );
}

bool XLifeModuleAppl :: IfHeight( void ) const
{
  if( OriginHeight == XLIFE_APPL_PARAM_DEFAULT ) {
    return( false );
  }

  return( true );
}

bool XLifeModuleAppl :: IfInitRandom( void ) const
{
  if( OriginInitRandom == XLIFE_APPL_PARAM_DEFAULT ) {
    return( false );
  }

  return( true );
}

const std::string XLifeModuleAppl :: GetMessageText( const int MsgNumber ) const
{
  switch( MsgNumber ) {
    case XLIFE_APPL_NO_ERR:
      return("No error.\n");
    case XLIFE_APPLICATION:
      return("Error in XLife module.\n");
    case XLIFE_APPL_START:
      return("  XLife program started.\n");
    case XLIFE_APPL_FINISH:
      return("  XLife program finished.\n");
    case XLIFE_APPL_MODULE:
      return("Init module.\n");
    case XLIFE_APPL_SHOW_PARAM:
      return("");
    default:;
  }

  return( APSModule :: GetMessageText( MsgNumber ) );
}

void XLifeModuleAppl :: PrintParameters( void ) const
{
  if( OriginBorderField == XLIFE_APPL_PARAM_MAIN_CONFIG ) {
    std::ostringstream Msg;
    Msg << "Parameter BorderField from file " << ProjectFilePath << ": " << BorderField << std::endl;
    InfoMessage( XLIFE_APPL_SHOW_PARAM, Msg.str() );
  }

  if( OriginDelta == XLIFE_APPL_PARAM_MAIN_CONFIG ) {
    std::ostringstream Msg;
    Msg << "Parameter Delta from file " << ProjectFilePath << ": " << Delta << std::endl;
    InfoMessage( XLIFE_APPL_SHOW_PARAM, Msg.str() );
  }

  if( OriginWidth == XLIFE_APPL_PARAM_MAIN_CONFIG ) {
    std::ostringstream Msg;
    Msg << "Parameter Width from file " << ProjectFilePath << ": " << Width << std::endl;
    InfoMessage( XLIFE_APPL_SHOW_PARAM, Msg.str() );
  }

  if( OriginHeight == XLIFE_APPL_PARAM_MAIN_CONFIG ) {
    std::ostringstream Msg;
    Msg << "Parameter Height from file " << ProjectFilePath << ": " << Height << std::endl;
    InfoMessage( XLIFE_APPL_SHOW_PARAM, Msg.str() );
  }

  if( OriginInitRandom == XLIFE_APPL_PARAM_MAIN_CONFIG ) {
    std::ostringstream Msg;
    Msg << "Parameter InitRandom from file " << ProjectFilePath << ": " << InitRandom << std::endl;
    InfoMessage( XLIFE_APPL_SHOW_PARAM, Msg.str() );
  }
}

int XLifeModuleAppl :: Run( void )
{
  APSReadConfig * pAPSReadConfig;
  XLifeConfig   * pXLifeConfig;
  int             RetCode = XLIFE_APPL_NO_ERR;

  ShowBanner();

  InfoMessage( XLIFE_APPL_START );

  if( ProjectFilePath != "" ) {
    pXLifeConfig = new XLifeConfig();

    pAPSReadConfig = new APSReadConfig( pXLifeConfigReadSubModule, ProjectFilePath );

    RetCode = pAPSReadConfig->Read( pXLifeConfig );

    if( !RetCode ) {
      if( !pXLifeConfig->GetBorderField( BorderField ) ) {
        OriginBorderField = XLIFE_APPL_PARAM_MAIN_CONFIG;
      }

      if( !pXLifeConfig->GetDelta( Delta ) ) {
        OriginDelta = XLIFE_APPL_PARAM_MAIN_CONFIG;
      }

      if( !pXLifeConfig->GetWidth( Width ) ) {
        OriginWidth = XLIFE_APPL_PARAM_MAIN_CONFIG;
      }

      if( !pXLifeConfig->GetHeight( Height ) ) {
        OriginHeight = XLIFE_APPL_PARAM_MAIN_CONFIG;
      }

      if( !pXLifeConfig->GetInitRandom( InitRandom ) ) {
        OriginInitRandom = XLIFE_APPL_PARAM_MAIN_CONFIG;
      }

      PrintParameters();
    }

    delete pAPSReadConfig;

    delete pXLifeConfig;
  }

  return( RetCode );
}

}}

//---------------------------- End of file ---------------------------


